use rumqttc::{AsyncClient, MqttOptions, QoS};
use std::{time::Duration};
use time::{format_description, OffsetDateTime};
use tokio::task;
use tracing::warn;

pub struct MqttConnection {
    pub client: AsyncClient,
}

impl MqttConnection {
    pub fn new() -> Self {
        let mut options = MqttOptions::new("rumqtt-async", "192.168.31.11", 1883);
        options.set_keep_alive(Duration::from_secs(5));

        let (mut client, mut eventloop) = AsyncClient::new(options, 10);
        task::spawn(async move {
            while let Ok(notification) = eventloop.poll().await {
                println!("Received = {:?}", notification);
            }
        });
        Self { client }
    }

    pub fn initialize(&mut self) {
        self.subscribe_board();
        self.broadcast_desktop_online();
    }

    async fn subscribe_board(&self) {
        self.client
            .subscribe("screen-bg-light/board/#", QoS::AtMostOnce).await;
    }

    fn broadcast_desktop_online(&mut self) {
        let client = self.client.to_owned();
        task::spawn(async move {
            loop {
                match OffsetDateTime::now_utc()
                    .format(&format_description::well_known::Iso8601::DEFAULT)
                {
                    Ok(now_str) => {
                        match client
                            .publish(
                                "screen-bg-light/desktop/online",
                                QoS::AtLeastOnce,
                                false,
                                now_str.as_bytes(),
                            )
                            .await
                        {
                            Ok(_) => {}
                            Err(error) => {
                                warn!("can not publish last online time. {}", error)
                            }
                        }
                    }
                    Err(error) => {
                        warn!("can not get time for now. {}", error);
                    }
                }
                tokio::time::sleep(Duration::from_millis(1000)).await;
            }
        });
    }
}
